import curses, time, locale
from curses import textpad

locale.setlocale(locale.LC_ALL, "")
code = locale.getpreferredencoding()


class Input:
    def __init__(self,
                 stdscr,
                 starty=0,
                 startx=0,
                 endy=0,
                 endx=0,
                 handle_submit=None):
        self.stdscr = stdscr
        self.handle_submit = handle_submit
        self.starty = starty + 2
        self.startx = startx + 2
        self.endy = endy - 1
        self.endx = endx - 2
        self.active = True
        scry, scrx = self.stdscr.getmaxyx()
        self.available_space = self.endx - self.startx
        self.win = None

    def render(self, status=None):
        curses.echo()
        curses.nocbreak()
        if self.win:
            del self.win
        self.win = curses.newwin(2, self.available_space - 2, self.starty,
                                 self.startx)
        self.box = textpad.Textbox(self.win, insert_mode=True)
        self.stdscr.refresh()
        self.contents = self.box.edit(self.__enter_is_terminate)
        del self.win
        self.win = None
        self.handle_submit(self.contents)
        curses.noecho()
        curses.cbreak()

    def receive_input(self, key):
        pass

    def __enter_is_terminate(self, x):
        if x == 10 or x == 13 or x == curses.KEY_ENTER:
            return 7

    def __printString(self, y, x, text, color):
        if color:
            self.stdscr.attron(curses.color_pair(color))
        self.stdscr.addstr(y, x, text)
        if color:
            self.stdscr.attroff(curses.color_pair(color))



import curses
from spotui.src.util import ms_to_hms, truncate
from spotui.src.menu import Menu
from spotui.src.component import Component


class NowPlaying(Component):
    def __init__(self, stdscr):
        self.stdscr = stdscr
        self.interactive = False
        self.restart()

    def restart(self):
        scry, scrx = self.stdscr.getmaxyx()
        self.startx = 0
        self.endx = scrx - 2
        self.starty = scry - 3
        self.endy = scry - 1
        self.component = NowPlayingComponent(
            self.stdscr,
            self.starty,
            self.startx,
            self.endy,
            self.endx,
        )

    def receive_input(self, key):
        pass


class NowPlayingComponent:
    def __init__(self, stdscr, starty, startx, endy, endx):
        self.stdscr = stdscr
        self.starty = starty
        self.startx = startx
        self.endy = endy
        self.endx = endx
        self.active = False
        self.playing = False
        self.track_name = "-"
        self.artist_name = "-"
        self.track_length = 0
        self.progress = 0
        self.progress_percent = 0

    def render(self, status):
        scry, scrx = self.stdscr.getmaxyx()
        if status:
            self.playing = status["is_playing"]
        if self.playing and status and status["item"]:
            current_track = status["item"]
            self.track_name = current_track["name"]
            self.artist_name = current_track["artists"][0]["name"]
            self.track_length = current_track["duration_ms"]
            self.progress = status["progress_ms"]
            self.progress_percent = ((self.progress / self.track_length) *
                                     100 if self.progress > 0
                                     and self.track_length > 0 else 0)
        shuffle = status["shuffle_state"] if status else False
        repeat = status["repeat_state"] if status else False
        shuffle_symbol = "咽" if shuffle else ""
        repeat_symbol = ""
        if repeat == "track":
            repeat_symbol = "綾"
        if repeat == "context":
            repeat_symbol = "凌"
        status_symbol = "契" if self.playing else " "
        timestamp = ms_to_hms(self.progress) + "/" + ms_to_hms(
            self.track_length)
        max_length = self.endx - self.startx - (len(timestamp) + 3)
        max_length = max_length if max_length > 0 else 0
        progress_length = round(
            ((self.endx - self.startx - 4) / 100) * self.progress_percent)
        track_info = truncate(
            ("Nothing is playing"
             if self.track_name == "-" else status_symbol + shuffle_symbol +
             repeat_symbol + " " + self.track_name + " - " + self.artist_name),
            max_length,
        )
        # Track + Artist
        self.stdscr.addstr(self.starty, self.startx + 1, track_info)

        # Progress text
        self.stdscr.addstr(self.starty, self.endx - len(timestamp), timestamp)

        # Progress bar
        for i in range(
                0,
                round(self.endx - self.startx - 4 - self.progress_percent) +
                1):
            self.stdscr.addstr(self.endy, self.endx - 2 - i, "░")
        self.stdscr.attron(curses.color_pair(11))
        self.stdscr.addstr(self.endy, self.startx + 1, "")
        for i in range(0, progress_length + 1):
            self.stdscr.addstr(self.endy, self.startx + 2 + i, "█")
        if self.progress_percent > 99.4:
            self.stdscr.addstr(self.endy, self.endx - 1, "")
        self.stdscr.attroff(curses.color_pair(11))
        if self.progress_percent < 99.5:
            self.stdscr.addstr(self.endy, self.endx - 1, "")



import curses


class Component:
    component = None
    startx = 0
    starty = 0
    endx = 0
    endy = 0
    title = None
    popup = False
    interactive = False

    def __init__(self, stdscr):
        self.stdscr = stdscr

    def activate(self):
        curses.curs_set(0)
        self.component.active = True

    def deactivate(self):
        curses.curs_set(1)
        self.component.active = False

    def create_border(self, color):
        self.stdscr.attron(curses.color_pair(color))

        # vertical borders
        for i in range(self.starty + 1, self.endy):
            self.stdscr.addstr(i, self.startx, "│")
            self.stdscr.addstr(i, self.endx, "│")

        # horizontal borders
        for i in range(self.startx + 1, self.endx):
            self.stdscr.addstr(self.starty, i, "─")
            self.stdscr.addstr(self.endy, i, "─")

        # top left corner
        self.stdscr.addstr(self.starty, self.startx, "╭")

        # top right corner
        self.stdscr.addstr(self.starty, self.endx, "╮")

        # bottom left corner
        self.stdscr.addstr(self.endy, self.startx, "╰")

        # bottom right corner
        self.stdscr.addstr(self.endy, self.endx, "╯")

        # title
        if self.title:
            self.stdscr.addstr(self.starty, self.startx + 2,
                               " " + self.title + " ")

        self.stdscr.attroff(curses.color_pair(color))

    def render(self, status=None):
        if self.popup:
            for x in range(self.startx, self.endx + 1):
                for y in range(self.starty, self.endy + 1):
                    self.stdscr.addstr(y, x, " ")
            self.create_border(10)
        elif self.interactive:
            self.create_border(5 if self.component.active else 4)
        self.component.render(status)



import curses
from spotui.src.util import truncate
from spotui.src.input import Input
from spotui.src.component import Component


class SearchInput(Component):
    def __init__(self, stdscr, api, handle_search, close):
        self.stdscr = stdscr
        self.api = api
        self.handle_search = handle_search
        self.close = close
        self.active = True
        self.popup = True
        self.title = "Search"
        self.interactive = False
        self.restart()

    def restart(self):
        scry, scrx = self.stdscr.getmaxyx()
        box_height = 4
        box_width = round(scrx / 3)
        self.startx = round((scrx / 2) - (box_width / 2))
        self.endx = self.startx + box_width
        self.starty = round((scry / 2) - (box_height / 2))
        self.endy = self.starty + box_height
        self.component = Input(
            self.stdscr,
            self.starty,
            self.startx,
            self.endy,
            self.endx,
            self.handle_search,
        )

    def activate(self):
        self.component.active = True

    def deactivate(self):
        self.component.active = False

    def receive_input(self, key):
        if key == curses.KEY_ENTER or key in [10, 13]:
            self.close()
        else:
            self.component.receive_input(key)



import curses


class Menu:
    def __init__(
        self,
        stdscr,
        items=[],
        starty=0,
        startx=0,
        endy=0,
        endx=0,
        active=False,
        selected=0,
        scroll_start=0,
    ):
        self.stdscr = stdscr
        self.starty = starty + 2
        self.startx = startx + 2
        self.endy = endy - 1
        self.endx = endx - 2
        self.items = items
        self.selected = selected
        self.active = active
        scry, scrx = self.stdscr.getmaxyx()
        self.available_space = self.endy - self.starty + 1
        self.scroll_start = scroll_start
        self.scroll_end = (len(items) -
                           1 if len(items) <= self.available_space +
                           self.scroll_start else self.available_space +
                           self.scroll_start - 1)

    def select(self, index):
        if index >= 0 and index <= len(self.items) - 1:
            self.selected = index

    def render(self, status=None):
        scry, scrx = self.stdscr.getmaxyx()
        for i, itemIndex in enumerate(
                range(self.scroll_start, self.scroll_end + 1)):
            item = self.items[itemIndex]
            x = self.startx
            y = self.starty + i
            selected = itemIndex == self.selected and self.active
            if y >= 0 and y <= scry and x >= 0 and x <= scrx:
                color = None
                if "highlight" in item and item["highlight"]:
                    color = 12
                if selected:
                    color = 6
                self.__printString(y, x, item["text"], color)

    def receive_input(self, key):
        if (key == curses.KEY_UP or key == ord("k")) and self.selected > 0:
            self.select(self.selected - 1)
        if (key == curses.KEY_DOWN
                or key == ord("j")) and self.selected < len(self.items) - 1:
            self.select(self.selected + 1)
        if self.selected < self.scroll_start:
            self.scroll_up()
        if self.selected > self.scroll_end:
            self.scroll_down()

    def scroll_up(self):
        self.scroll_start -= 1
        self.scroll_end -= 1

    def scroll_down(self):
        self.scroll_start += 1
        self.scroll_end += 1

    def __printString(self, y, x, text, color):
        if color:
            self.stdscr.attron(curses.color_pair(color))
        self.stdscr.addstr(y, x, text)
        if color:
            self.stdscr.attroff(curses.color_pair(color))
